==> nutrient-2022-0.1.20.tgz: package/src/patch.ts <==
import { getComponent } from "./component_manager.js";
import Module from "./module.js";
import Node, { MetadataMap, KeyValueMap } from "./node.js";
import observer from "./observer.js";
import Group from "./group.js";
import { Events } from "./events.js";

export type NodeMap = {
    [nodeId: number]: Node | Module;
}

export type GroupMap = {
    [groupId: number]: Group;
}

let patchIdCounter = 0;

class Patch {
    id = ++patchIdCounter;
    nodes: NodeMap = {};
    ownerModuleNode: Node | Module | null = null;
    groups: GroupMap = {};

    constructor(ownerNode: Node | Module | null) {
        this.ownerModuleNode = ownerNode;
    }

    isModule() {
        return this.ownerModuleNode != null;
    }

    createGroup(id: number, name: string, comment: string = "", nodeIdList: number[] = []) {
        let group = new Group(id, name || "New Group", comment, nodeIdList);
        this.groups[group.id] = group;
        observer.triggerEvent<Group>(Events.OnCreateGroup, group);
        return group;
    }

    removeGroup(id: number) {
        delete this.groups[id];
        observer.triggerEvent<number>(Events.OnRemoveGroup, id);
    }

    createNode(id: number, componentIdentKey: string, properties?: KeyValueMap, metadata?: MetadataMap): Node | Module | null {
        if (getComponent(componentIdentKey) == null) {
            return null;
        }
        //let id = IDGenerator.next();
        let node: Node | Module;
        if (componentIdentKey == "core.module") {
            node = this.nodes[id] = new Module(this, id, componentIdentKey);
        }
        else {
            node = new Node(this, id, componentIdentKey);
            if (componentIdentKey == "core.input" || componentIdentKey == "core.output") {
                if (this.ownerModuleNode != null) {
                    if (componentIdentKey == "core.input") {
                        (this.ownerModuleNode as Module).inputNodes[id] = node;
                    }
                    else if (componentIdentKey == "core.output") {
                        (this.ownerModuleNode as Module).outputNodes[id] = node;
                    }
                }
                else {
                    return null;
                }
            }
            this.nodes[id] = node;
        }
        node.properties = properties || {};
        node.metadata = metadata || {};
        if (node != null) {
            observer.triggerEvent<Node | Module>(Events.OnCreateNode, node);
        }
        return node;
    }


    removeNode(nodeId: number, removePorts = true) {
        let node = this.nodes[nodeId];
        if (node != null) {
            if (removePorts) {
                for (let portKey of Object.keys(node.outputs)) {
                    node.removeOutputConnection(portKey, null);
                }
                for (let portKey of Object.keys(node.inputs)) {
                    node.removeInputConnection(portKey, null);
                }
            }
            delete this.nodes[nodeId];
            observer.triggerEvent<Node | Module>(Events.OnRemoveNode, node);
        }
        return node;
    }

    getNode(nodeId: number) {
        let node = this.nodes[nodeId];
        return node;
    }

    findConnection(connectionId: number) {
        for (let node of Object.values(this.nodes)) {
            for (let port of Object.values(node.outputs)) {
                for (let [id, connection] of port.connections) {
                    if (id == connectionId) {
                        return connection;
                    }
                }
            }
        }
        return null;
    }

}

export default Patch;

==> nutrient-2022-0.1.26.tgz: package/src/patch.ts <==
import { getComponent } from "./component_manager.js";
import Module from "./module.js";
import Node, { MetadataMap, KeyValueMap } from "./node.js";
import observer from "./observer.js";
import Group from "./group.js";
import { Events } from "./events.js";

export type NodeMap = {
    [nodeId: number]: Node | Module;
}

export type GroupMap = {
    [groupId: number]: Group;
}

let patchIdCounter = 0;

class Patch {
    id = ++patchIdCounter;
    nodes: NodeMap = {};
    ownerModuleNode: Node | Module | null = null;
    groups: GroupMap = {};

    constructor(ownerNode: Node | Module | null) {
        this.ownerModuleNode = ownerNode;
    }

    isModule() {
        return this.ownerModuleNode != null;
    }

    createGroup(id: number, name: string, comment: string = "", nodeIdList: number[] = []) {
        let group = new Group(id, name || "New Group", comment, [...nodeIdList]);
        this.groups[group.id] = group;
        observer.triggerEvent<Group>(Events.OnCreateGroup, group);
        return group;
    }

    removeGroup(id: number) {
        let group = this.groups[id];
        delete this.groups[id];
        observer.triggerEvent<number>(Events.OnRemoveGroup, id);
        return group;
    }

    createNode(id: number, componentIdentKey: string, properties?: KeyValueMap, metadata?: MetadataMap): Node | Module | null {
        if (getComponent(componentIdentKey) == null) {
            return null;
        }
        //let id = IDGenerator.next();
        let node: Node | Module;
        if (componentIdentKey == "core.module") {
            node = this.nodes[id] = new Module(this, id, componentIdentKey);
        }
        else {
            node = new Node(this, id, componentIdentKey);
            if (componentIdentKey == "core.input" || componentIdentKey == "core.output") {
                if (this.ownerModuleNode != null) {
                    if (componentIdentKey == "core.input") {
                        (this.ownerModuleNode as Module).inputNodes[id] = node;
                    }
                    else if (componentIdentKey == "core.output") {
                        (this.ownerModuleNode as Module).outputNodes[id] = node;
                    }
                }
                else {
                    return null;
                }
            }
            this.nodes[id] = node;
        }
        node.properties = properties || {};
        node.metadata = metadata || {};
        if (node != null) {
            observer.triggerEvent<Node | Module>(Events.OnCreateNode, node);
        }
        return node;
    }

    removeNode(nodeId: number, removePorts = true) {
        let node = this.nodes[nodeId];
        let removedFromGroupId: number | null = null;
        let removedEmptyGroup: Group | null = null;
        if (node != null) {
            if (removePorts) {
                for (let portKey of Object.keys(node.outputs)) {
                    node.removeOutputConnection(portKey, null);
                }
                for (let portKey of Object.keys(node.inputs)) {
                    node.removeInputConnection(portKey, null);
                }
            }
            for (let group of Object.values(this.groups)) {
                if (group.hasNodeId(nodeId)) {
                    group.removeNodeId(nodeId);
                    removedFromGroupId = group.id;
                    if (group.isEmpty()) {
                        this.removeGroup(group.id);
                        removedEmptyGroup = group;
                    }
                    break;
                }
            }
            delete this.nodes[nodeId];
            observer.triggerEvent<Node | Module>(Events.OnRemoveNode, node);
        }
        return {
            removedFromGroupId: removedFromGroupId,
            removedEmptyGroup: removedEmptyGroup
        };
    }

    getNode(nodeId: number) {
        let node = this.nodes[nodeId];
        return node;
    }

    findConnection(connectionId: number) {
        for (let node of Object.values(this.nodes)) {
            for (let port of Object.values(node.outputs)) {
                for (let [id, connection] of port.connections) {
                    if (id == connectionId) {
                        return connection;
                    }
                }
            }
        }
        return null;
    }

    mapNode(node: Node) {
        this.nodes[node.id] = node;
        observer.triggerEvent(Events.OnMoveNodeFromModule, node);
    }

}

export default Patch;
